import {
  createContext,
  createSignal,
  onCleanup,
  onMount,
  useContext,
} from "solid-js";

const CooldownContext = createContext();

export default function CooldownProvider(props: any) {
  const [cooldown, setCooldown] = createSignal<{ [id: string]: number }>({});

  const store = [
    cooldown,
    {
      freeze(id: string, duration: number) {
        setCooldown((o) => {
          o[id] = duration;
          return o;
        });
      },
      unfreeze(id: string) {
        setCooldown((o) => {
          o[id] = 0;
          return o;
        });
      },
    },
  ];

  function reduceCooldown(amount: number = 1000) {
    setCooldown((o) => {
      Object.keys(o).forEach((k) => {
        if (o[k] > 0) {
          o[k] -= amount;
        }
      });

      return o;
    });
  }

  onMount(() => {
    const loop = setInterval(() => reduceCooldown(10), 10);
    onCleanup(() => clearInterval(loop));
  });

  return (
    <CooldownContext.Provider value={store}>
      {props.children}
    </CooldownContext.Provider>
  );
}

export function useCooldown(): any[2] {
  return useContext(CooldownContext);
}
